/**
 * 
 */
package com.example.layouttest.presenter;

import com.example.layouttest.view.View;
import com.example.layouttest.view.ViewObserver;

/**
 * Abstract {@link Presenter} class to manage view creation.
 * 
 * @author Richard Gal
 * 
 * @param <V>
 *            The type of the {@link View} that this presenter controls.
 * @param <P>
 *            The type of the {@link ViewObserver} that this presenter
 *            implements to observe view events.
 */
public abstract class AbstractPresenter<V extends View<P>, P extends ViewObserver> 
    implements Presenter, ViewObserver {

    private V view;
    
    /**
     * Creates the concrete {@link View} that this presenter controls. The
     * typical use is
     * <ul>
     * <li>to implement this method in a production presenter class by
     *     instantiating the concrete {@link View}</li>
     * <li>to use this method with Spring's method injection and inject a mock
     *     view in a test harness.</li>
     * </ul>
     * 
     * @return The concrete {@link View} that this presenter controls.
     */
    protected abstract V createView();
    
    /**
     * Returns the {@link View} that this presenter controls. It also
     * instantiates and initializes the view lazily if needed.
     * 
     * @return the {@link View} that this presenter controls.
     */
    @SuppressWarnings("unchecked")
    public V getView() {
        if (view == null) {
            view = createView();
            view.setPresenter((P)this);
        } 
        return view;
    } 

}
